<?php

namespace Drupal\vipps_mobilepay_commerce\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\vipps_mobilepay\Entity\VippsMobilepayConnection;

/**
 * The payment gateway base class.
 */
abstract class VippsMobilePayPaymentGatewayBase extends OffsitePaymentGatewayBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'connection_profile' => NULL,
      'prefix' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Hide this value for all Vipps MobilePay gateways.
    $form['mode']['#access'] = FALSE;

    $options = $this->entityTypeManager->getStorage('vipps_mobilepay_connection')->loadMultiple();
    $options = array_map(function (VippsMobilepayConnection $profile) {
      return $profile->status() ? $profile->label() : $this->t('@label (disabled)', ['@label' => $profile->label()]);
    }, $options);

    $form['connection_profile'] = [
      '#type' => 'select',
      '#title' => $this->t('Vipps MobilePay Connection Profile'),
      '#required' => TRUE,
      '#description' => $this->t('Choose one of the connection profiles defined on <a href=:url>this page</a>', [':url' => Url::fromRoute('entity.vipps_mobilepay_connection.collection')->toString()]),
      '#default_value' => $this->configuration['connection_profile'],
      '#options' => $options,
    ];
    $form['prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Prefix'),
      '#description' => $this->t("In case you're creating payments from multiple independent systems using the same connection credentials, add alphanumeric prefix to reference in Vipps MobilePay"),
      '#default_value' => $this->configuration['prefix'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['connection_profile'] = $values['connection_profile'];
      $this->configuration['prefix'] = $values['prefix'];
    }
  }

  /**
   * Returns the connection profile.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The profile.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getConnectionProfile(): ?VippsMobilepayConnection {
    if (!isset($this->configuration['connection_profile'])) {
      return NULL;
    }
    return $this->entityTypeManager->getStorage('vipps_mobilepay_connection')->load($this->configuration['connection_profile']);
  }

}
