<?php

namespace Drupal\vipps_mobilepay_commerce\Plugin\Commerce\PaymentMethodType;

use Drupal\commerce_payment\Entity\PaymentMethodInterface;
use Drupal\commerce_payment\FailedPaymentDetailsInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentMethodType\PaymentMethodTypeBase;
use Drupal\entity\BundleFieldDefinition;

/**
 * Provides the wallet payment method type.
 *
 * @CommercePaymentMethodType(
 *   id = "vipps_mobilepay_wallet",
 *   label = @Translation("Wallet"),
 * )
 */
class VippsMobilePayWallet extends PaymentMethodTypeBase implements FailedPaymentDetailsInterface {

  /**
   * {@inheritdoc}
   */
  public function buildLabel(PaymentMethodInterface $payment_method) {

    $country_code = $payment_method->vipps_mobilepay_country_code->value;
    $phone_number = $payment_method->vipps_mobilepay_telephone->value;
    $args = [
      '@type' => $country_code === 'NO' ? 'Vipps' : 'MobilePay',
      '@phone_number' => $phone_number,
    ];
    return $this->t('@type agreement for @phone_number', $args);
  }

  /**
   * {@inheritdoc}
   */
  public function failedPaymentDetails(PaymentMethodInterface $payment_method): array {
    return ['agreement_id' => $payment_method->getRemoteId()];
  }

  /**
   * {@inheritdoc}
   */
  public function buildFieldDefinitions() {
    $fields = parent::buildFieldDefinitions();

    $fields['vipps_mobilepay_country_code'] = BundleFieldDefinition::create('string')
      ->setLabel(t('Country Code'))
      ->setDescription(t('The country code associated with the Agreement.'))
      ->setRequired(TRUE);
    $fields['vipps_mobilepay_price'] = BundleFieldDefinition::create('commerce_price')
      ->setLabel(t('Agreement Price'))
      ->setDescription(t('The can be the price of agreement or the max amount chosen by the customer.'))
      ->setRequired(TRUE);
    $fields['vipps_mobilepay_telephone'] = BundleFieldDefinition::create('telephone')
      ->setLabel(t('Telephone number'))
      ->setDescription(t('The telephone number associated with the Vipps MobilePay account.'))
      ->setRequired(TRUE);
    $fields['vipps_mobilepay_sub'] = BundleFieldDefinition::create('string')
      ->setLabel(t('Vipps MobilePay User Identifier'))
      ->setDescription(t("The Agreement's user identifier."))
      ->setRequired(TRUE);
    $fields['vipps_mobilepay_agreement_name'] = BundleFieldDefinition::create('string')
      ->setLabel(t('Product Name'))
      ->setDescription(t('The Product Name for the Agreement.'))
      ->setRequired(TRUE);
    $fields['vipps_mobilepay_agreement_description'] = BundleFieldDefinition::create('string')
      ->setLabel(t('Product Description'))
      ->setDescription(t('The Product Description for the Agreement.'))
      ->setRequired(TRUE);

    return $fields;
  }

}
