<?php

namespace Drupal\vipps_mobilepay_commerce\Resolver;

use Drupal\commerce_payment\Entity\PaymentInterface;

/**
 * Chain Reference Resolver.
 */
class ChainReferenceResolver implements ChainReferenceResolverInterface {

  /**
   * The resolvers.
   *
   * @var \Drupal\vipps_mobilepay_commerce\Resolver\ReferenceResolverInterface[]
   */
  protected $resolvers = [];

  /**
   * Constructs a new ChainReferenceResolver object.
   *
   * @param \Drupal\vipps_mobilepay_commerce\Resolver\ReferenceResolverInterface[] $resolvers
   *   The resolvers.
   */
  public function __construct(array $resolvers = []) {
    $this->resolvers = $resolvers;
  }

  /**
   * {@inheritdoc}
   */
  public function addResolver(ReferenceResolverInterface $resolver) {
    $this->resolvers[] = $resolver;
  }

  /**
   * {@inheritdoc}
   */
  public function getResolvers() {
    return $this->resolvers;
  }

  /**
   * {@inheritdoc}
   */
  public function resolve(PaymentInterface $payment): ?string {
    foreach ($this->resolvers as $resolver) {
      $result = $resolver->resolve($payment);
      if ($result) {
        return $result;
      }
    }
    throw new \LogicException('No resolver found for payment ' . $payment->id());
  }

}
