<?php

namespace Drupal\vipps_mobilepay_commerce\Service;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryOverrideInterface;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\State\StateInterface;

/**
 * Overrides default Commerce Payment Gateway config.
 */
class ConfigOverrider implements ConfigFactoryOverrideInterface {

  const STATE_KEY = 'vipps_mobilepay_gateway_mode_overrides';

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  public function __construct(StateInterface $state) {
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public function loadOverrides($names) {
    $overrides = [];
    $config_overrides = $this->state->get(static::STATE_KEY) ?? [];
    if (array_intersect(array_keys($config_overrides), $names)) {
      foreach ($names as $name) {
        if (!isset($config_overrides[$name])) {
          continue;
        }
        $overrides += [
          $name => [
            'configuration' => [
              'mode' => $config_overrides[$name],
            ],
          ],
        ];
      }
    }

    return $overrides;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheSuffix() {
    return 'ConfigOverrider';
  }

  /**
   * {@inheritdoc}
   */
  public function createConfigObject($name, $collection = StorageInterface::DEFAULT_COLLECTION) {
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheableMetadata($name) {
    return new CacheableMetadata();
  }

}
